import datetime

from rest_framework import serializers, status
from rest_framework.exceptions import ValidationError

from dailydata.models import DailyData
from flocks.models import Breed, Flock, Integrator, sys_choices

boolean_choice = [(True, True), (False, False)]


class BaseSerializer(serializers.Serializer):
    def is_valid(self, raise_exception=False):
        assert not hasattr(self, "restore_object"), (
                "Serializer `%s.%s` has old-style version 2 `.restore_object()` "
                "that is no longer compatible with REST framework 3. "
                "Use the new-style `.create()` and `.update()` methods instead."
                % (self.__class__.__module__, self.__class__.__name__)
        )

        assert hasattr(self, "initial_data"), (
            "Cannot call `.is_valid()` as no `data=` keyword argument was "
            "passed when instantiating the serializer instance."
        )

        if not hasattr(self, "_validated_data"):
            try:
                self._validated_data = self.run_validation(self.initial_data)
            except ValidationError as exc:
                self._validated_data = {}
                self._errors = exc.detail
            else:
                self._errors = {}

        if self._errors and raise_exception:
            "Invalid parameters."
            for key, value in self.errors.items():
                if key == "non_field_errors" and self.data["mobile_or_email"]:
                    key = "country_code"
                elif key == "non_field_errors" and not self.data["mobile_or_email"]:
                    key = self.data["type"]
                f"{str(key.replace('_', ' '))} {str(value[0])}"
                break
            raise ValidationError(
                {
                    "message": "Validation Error",
                    "result": self.errors,
                    "status": status.HTTP_400_BAD_REQUEST,
                }
            )
        return not bool(self._errors)


class AddFlockSerializer(BaseSerializer):
    """
    Class to validate data for creating flock object.
    """

    farm_public_id = serializers.IntegerField()
    flock_name = serializers.CharField(max_length=255)
    integration = serializers.BooleanField()
    hatch_date = serializers.DateField()
    flock_size = serializers.IntegerField()
    actual_birds = serializers.IntegerField()
    breed_public_id = serializers.IntegerField()
    hatchery_public_id = serializers.IntegerField(required=False)
    hatchery_name = serializers.CharField(max_length=555)
    chick_weight = serializers.IntegerField()
    hatchery_vaccine = serializers.IntegerField(required=False)
    feed_sys = serializers.ChoiceField(choices=sys_choices)
    drinking_sys = serializers.ChoiceField(choices=sys_choices)
    ec_shed = serializers.BooleanField()

    def __init__(self, *args, **kwargs):
        super(AddFlockSerializer, self).__init__(*args, **kwargs)
        if kwargs.get("data").get("integration"):
            self.fields["integrator_public_id"] = serializers.IntegerField(
                allow_null=True
            )
        else:
            self.fields["chick_cost"] = serializers.IntegerField(allow_null=True)


class BreedListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Breed
        fields = ["name", "public_id"]


class GetFlockSerializer(serializers.ModelSerializer):
    flock_name = serializers.CharField(max_length=255, source="name")
    flock_public_id = serializers.CharField(max_length=255, source="public_id")
    chick_breed = serializers.CharField(max_length=255, source="breed.name")
    integrator_name = serializers.CharField(
        max_length=255, source="integrator.name", allow_null=True
    )
    integrator_public_id = serializers.IntegerField(
        source="integrator.public_id", allow_null=True
    )
    chick_breed_public_id = serializers.IntegerField(
        source="breed.public_id", allow_null=True
    )
    hatchery_name = serializers.CharField(max_length=555)
    farm_public_id = serializers.IntegerField(source="farm.public_id")
    farm_name = serializers.CharField(source="farm.name", max_length=255)
    age = serializers.SerializerMethodField()
    address = serializers.CharField(max_length=255, source='farm.address')
    mobile_number = serializers.IntegerField(source='user.mobile_number')

    class Meta:
        model = Flock
        fields = [
            "farm_public_id",
            "farm_name",
            "flock_public_id",
            "flock_name",
            "hatch_date",
            "flock_size",
            "actual_birds",
            "chick_breed",
            "chick_breed_public_id",
            "hatchery_name",
            "integration",
            "integrator_name",
            "integrator_public_id",
            "chick_weight",
            "ec_shed",
            "feed_sys",
            "chick_cost",
            "drinking_sys",
            "age",
            "address",
            "mobile_number"
        ]

    def get_age(self, obj):
        date = datetime.datetime.today().date()
        age = date - obj.hatch_date
        return age.days


class IntegratorListSerializer(serializers.ModelSerializer):
    class Meta:
        model = Integrator
        fields = ["public_id", "name", "nickname"]


class UpdateFlockSerializer(serializers.Serializer):
    flock_name = serializers.CharField(max_length=255, required=False)
    hatch_date = serializers.DateField(required=False)
    flock_size = serializers.IntegerField(required=False)
    actual_birds = serializers.IntegerField(required=False)
    breed_public_id = serializers.IntegerField(required=False)
    hatchery = serializers.CharField(max_length=255, required=False)
    hatchery_name = serializers.CharField(max_length=255, required=False)
    integration = serializers.BooleanField(required=False)
    integrator_public_id = serializers.IntegerField(required=False)
    chick_weight = serializers.IntegerField(required=False)
    ec_shed = serializers.BooleanField(required=False)
    feed_sys = serializers.ChoiceField(choices=sys_choices, required=False)
    drinking_sys = serializers.ChoiceField(choices=sys_choices, required=False)

    def __init__(self, *args, **kwargs):
        super(UpdateFlockSerializer, self).__init__(*args, **kwargs)
        if kwargs.get("data").get("integration") is True:
            self.fields["integrator_public_id"] = serializers.IntegerField(
                allow_null=True
            )
        else:
            self.fields["chick_cost"] = serializers.IntegerField(allow_null=True)


class GetDailyDataDetailSerializer(serializers.ModelSerializer):
    class Meta:
        model = DailyData
        fields = ["public_id", "date"]
